package demo.searchfield;

import java.awt.Component;
import java.awt.EventQueue;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.util.ArrayList;

import javax.swing.AbstractAction;
import javax.swing.JFrame;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.UIManager;

import de.javasoft.swing.JYPopupMenu;
import de.javasoft.swing.JYSearchField;
import de.javasoft.swing.plaf.jysearchfield.SearchActionEvent;
import de.javasoft.swing.plaf.jysearchfield.SearchActionEvent.Initiator;

public class SearchFieldHistoryDemo extends JFrame
{
  public static void main(String[] args) throws Exception
  {
    UIManager.setLookAndFeel("de.javasoft.plaf.synthetica.SyntheticaClassyLookAndFeel");
    
    EventQueue.invokeLater(new Runnable()
    {      
      public void run()
      {
        new SearchFieldHistoryDemo();
      }
    });
  }
  
  public SearchFieldHistoryDemo()
  {
    setLayout(new FlowLayout());
    
    JYSearchField searchField = new HistorySearchField(20);
    searchField.setSearchPopupButtonEnabled(true);
    searchField.setSearchAction(new SearchAction());
    JYPopupMenu popup = new JYPopupMenu();
    popup.setVisibleElements(10);
    searchField.setSearchPopupMenu(popup);    
    
    add(searchField);
    
    setDefaultCloseOperation(EXIT_ON_CLOSE);
    setTitle(getClass().getSimpleName());
    setSize(450, 340);
    setLocationRelativeTo(null);
    setVisible(true);
  }
  
  
  private static class SearchAction extends AbstractAction
  {
    public void actionPerformed(ActionEvent evt)
    {
      //do not execute on button click
      if (((SearchActionEvent)evt).getInitiator() == Initiator.SEARCH_POPUP_BUTTON)
        return;      
      
      HistorySearchField searchField = (HistorySearchField)evt.getSource();
      searchField.updateHistory();
      String searchText = searchField.getText();
      System.out.println("Do search action for: " + searchText);
    }    
  }
  
  
  private static class HistorySearchField extends JYSearchField
  {
    private static int maxItems = 50;

    public HistorySearchField(int columns)
    {
      super(columns);
    }
    
    public void updateHistory()
    {
      String searchText = getText().trim();
      if (searchText.length() == 0)
        return;
      
      //cache items
      JYPopupMenu popup = (JYPopupMenu)getSearchPopupMenu();
      JPanel p = popup.getMenuElementPanel();
      ArrayList<JMenuItem> itemList = new ArrayList<JMenuItem>();
      for (Component c : p.getComponents())
        if (c instanceof JMenuItem)
          itemList.add((JMenuItem)c);
      p.removeAll();
      
      //add new item at top
      JMenuItem newItem = new JMenuItem(new AbstractAction(searchText)
      {      
        public void actionPerformed(ActionEvent evt)
        {
          setText((String)getValue(NAME)); 
          getSearchAction().actionPerformed(new SearchActionEvent(HistorySearchField.this, ActionEvent.ACTION_PERFORMED, "", System.currentTimeMillis(), 0, Initiator.SEARCH_FIELD));
        }
      });      
      popup.add(newItem);
      
      //add old items
      for (JMenuItem mi : itemList)
      {
        if (p.getComponentCount() >= maxItems)
          break;
        if (!searchText.equals(mi.getText()))
          popup.add(mi);
      }
    }     
  }
}
